@@ -7,7 +7,7 @@
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
+** the Free Software Foundation; either version 3 of the License, or
 ** (at your option) any later version.
 ** 
 ** This program is distributed in the hope that it will be useful,
@@ -29,6 +29,7 @@
 **           GNU Project
 **
 */
+#include "ac_config.h"
 
 /*
 #define MESSAGES
@@ -86,7 +87,7 @@ static ProcessData gpd = NULL;
 /* process_get_pd */
 /*------------------------------------------------------------*/
 
-static ProcessData process_get_pd()
+static ProcessData process_get_pd(void)
 {
    ProcessData pd;
    Cardinal size = sizeof(ProcessDataStruct);
