@@ -66,9 +66,15 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include <string.h>
 
+#ifdef HAVE_ZIO
+# include <zio.h>
+#endif
+
 #include <sys/stat.h>
 
 #ifndef SEEK_SET
@@ -78,6 +84,7 @@
 #define BUFSIZ 1024
 #endif
 #include <ctype.h>
+#include <sys/types.h>
 
 #include "paths.h"
 #include INC_X11(Intrinsic.h)
@@ -99,6 +106,10 @@ extern int gv_infoSkipErrors;
 #define memset(a,b,c) bzero(a,c)
 #endif
 
+extern int sec_sscanf(const char *, const char *, ...);
+
+
+
 /* We use this helper function for providing proper */
 /* case and colon :-) insensitive DSC matching */
 static int dsc_strncmp(s1, s2, n)
@@ -392,6 +403,7 @@ psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_
 	cmd_uncompress=NULL;
       }
     }
+#ifndef HAVE_ZIO
     if (cmd_uncompress) {
       struct document *retval = NULL;
       FILE *tmpfile = (FILE*)NULL;
@@ -445,6 +457,23 @@ unc_ok:
       *filename_uncP = (char*)GV_XtNewString(filename_unc);
       goto unc_ok;
     }
+#else
+    if (cmd_uncompress) {
+      FILE *zfile = fzopen(filename, "r");
+      INFMESSAGE(is compressed)
+      if (!zfile) {
+	char s[512];
+	sprintf(s,"Uncompressing of\n%s\nfailed.",filename);
+	NotePopupShowMessage(s);
+	ENDMESSAGE(psscan)
+	return(NULL);
+      }
+      fclose(*fileP);
+      *fileP = zfile;
+      cmd_uncompress = NULL;
+      *filename_uncP = NULL;
+    }
+#endif
 
     respect_eof = (scanstyle & SCANSTYLE_IGNORE_EOF) ? 0 : 1;
     ignore_dsc = (scanstyle & SCANSTYLE_IGNORE_DSC) ? 1 : 0;
@@ -473,7 +502,7 @@ unc_ok:
       CHECK_MALLOCED(doc);
       memset(doc, 0, sizeof(struct document));
       *text=0;
-      sscanf(line, "%*s %256s", text);
+      sec_sscanf(line, "%*s %256s", text);
       /*###jp###*/
       /*doc->epsf = iscomment(text, "EPSF-");*/
       doc->epsf = iscomment(text, "EPSF");
@@ -490,8 +519,8 @@ unc_ok:
       char cmd[512];
       char s[512];
       mode_t old_umask;
-      FILE* tmp_file;
       String tmp_filename;
+      int tmpfd, tmp_fd;
 
       filename_dsc=file_getTmpFilename(NULL,filename_raw);
       /*      sprintf(cmd,cmd_scan_pdf,filename,filename_dsc); */
@@ -535,21 +564,24 @@ unc_ok:
       INFSMESSAGE(scan command,cmd)
       
       tmp_filename=file_getTmpFilename(NULL,filename_raw);
-      tmp_file = fopen(tmp_filename, "w");
-      dup2(2,100);
-      close(2); dup2( fileno(tmp_file), 2);
+
+      tmp_fd = open(tmp_filename, O_WRONLY|O_CREAT|O_TRUNC, 0666);
+      tmpfd = dup(2);
+      close(2); dup2( tmp_fd, 2);
       
 
       if (system(cmd) || file_fileIsNotUseful(filename_dsc)) {
         char line[1000];
 	int found;
+	FILE* tmp_file;
+	
 	found = 0;
       
 	INFMESSAGE(scan subprocess failed)
 	close(2);
-	dup2(100,2);
-	close(100);
-	fclose(tmp_file);
+	dup2(tmpfd,2);
+	close(tmpfd);
+	close(tmp_fd);
 	
         tmp_file = fopen( tmp_filename, "r" );
 	while ( fgets( line, 999, tmp_file) )
@@ -587,9 +619,10 @@ scan_ok:
       }
 
       close(2);
-      dup2(100,2);
-      close(100);
-      fclose(tmp_file);
+      dup2(tmpfd,2);
+      close(tmpfd);
+      close(tmp_fd);
+      
       unlink((char*) tmp_filename);
       GV_XtFree(tmp_filename);
 
@@ -628,11 +661,11 @@ scan_ok:
 	} else if (doc->date == NULL && iscomment(line+2, "CreationDate:")) {
 	    doc->date = gettextline(line+length("%%CreationDate:"));
 	} else if (bb_set == NONE && iscomment(line+2, "BoundingBox:")) {
-	    sscanf(line+length("%%BoundingBox:"), "%256s", text);
+	    sec_sscanf(line+length("%%BoundingBox:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		bb_set = ATEND;
 	    } else {
-		if (sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
+		if (sec_sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
 			   &(doc->boundingbox[LLX]),
 			   &(doc->boundingbox[LLY]),
 			   &(doc->boundingbox[URX]),
@@ -640,7 +673,7 @@ scan_ok:
 		    bb_set = 1;
 		else {
 		    float fllx, flly, furx, fury;
-		    if (sscanf(line+length("%%BoundingBox:"), "%f %f %f %f",
+		    if (sec_sscanf(line+length("%%BoundingBox:"), "%f %f %f %f",
 			       &fllx, &flly, &furx, &fury) == 4) {
 			bb_set = 1;
 			doc->boundingbox[LLX] = fllx;
@@ -660,7 +693,7 @@ scan_ok:
 	    }
 	} else if (orientation_set == NONE &&
 		   iscomment(line+2, "Orientation:")) {
-	    sscanf(line+length("%%Orientation:"), "%256s", text);
+	    sec_sscanf(line+length("%%Orientation:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		orientation_set = ATEND;
 	    } else if (strcmp(text, "Portrait") == 0) {
@@ -671,7 +704,7 @@ scan_ok:
 		orientation_set = 1;
 	    }
 	} else if (page_order_set == NONE && iscomment(line+2, "PageOrder:")) {
-	    sscanf(line+length("%%PageOrder:"), "%256s", text);
+	    sec_sscanf(line+length("%%PageOrder:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		page_order_set = ATEND;
 	    } else if (strcmp(text, "Ascend") == 0) {
@@ -685,11 +718,11 @@ scan_ok:
 		page_order_set = 1;
 	    }
 	} else if (pages_set == NONE && iscomment(line+2, "Pages:")) {
-	    sscanf(line+length("%%Pages:"), "%256s", text);
+	    sec_sscanf(line+length("%%Pages:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		pages_set = ATEND;
 	    } else {
-		switch (sscanf(line+length("%%Pages:"), "%d %d",
+		switch (sec_sscanf(line+length("%%Pages:"), "%d %d",
 			       &maxpages, &i)) {
 		    case 2:
 			if (page_order_set == NONE) {
@@ -721,7 +754,7 @@ scan_ok:
 	    doc->media[0].name = ps_gettext(line+length("%%DocumentMedia:"),
 					    &next_char);
 	    if (doc->media[0].name != NULL) {
-		if (sscanf(next_char, "%f %f", &w, &h) == 2) {
+		if (sec_sscanf(next_char, "%f %f", &w, &h) == 2) {
 		    doc->media[0].width = w + 0.5;
 		    doc->media[0].height = h + 0.5;
 		}
@@ -742,7 +775,7 @@ scan_ok:
 		doc->media[doc->nummedia].name = ps_gettext(line+length("%%+"),
 							    &next_char);
 		if (doc->media[doc->nummedia].name != NULL) {
-		    if (sscanf(next_char, "%f %f", &w, &h) == 2) {
+		    if (sec_sscanf(next_char, "%f %f", &w, &h) == 2) {
 			doc->media[doc->nummedia].width = w + 0.5;
 			doc->media[doc->nummedia].height = h + 0.5;
 		    }
@@ -905,7 +938,7 @@ scan_ok:
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%256s", text);
+		sec_sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->default_page_orientation = PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -924,7 +957,7 @@ scan_ok:
 		PS_free(cp);
 	    } else if (page_bb_set == NONE &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		if (sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
+		if (sec_sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
 			   &(doc->default_page_boundingbox[LLX]),
 			   &(doc->default_page_boundingbox[LLY]),
 			   &(doc->default_page_boundingbox[URX]),
@@ -932,7 +965,7 @@ scan_ok:
 		    page_bb_set = 1;
 		else {
 		    float fllx, flly, furx, fury;
-		    if (sscanf(line+length("%%PageBoundingBox:"), "%f %f %f %f",
+		    if (sec_sscanf(line+length("%%PageBoundingBox:"), "%f %f %f %f",
 			       &fllx, &flly, &furx, &fury) == 4) {
 			page_bb_set = 1;
 			doc->default_page_boundingbox[LLX] = fllx;
@@ -1027,7 +1060,7 @@ scan_ok:
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%256s", text);
+		sec_sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->default_page_orientation = PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -1050,7 +1083,7 @@ scan_ok:
 		PS_free(cp);
 	    } else if (page_bb_set == NONE &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		if (sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
+		if (sec_sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
 			   &(doc->default_page_boundingbox[LLX]),
 			   &(doc->default_page_boundingbox[LLY]),
 			   &(doc->default_page_boundingbox[URX]),
@@ -1058,7 +1091,7 @@ scan_ok:
 		    page_bb_set = 1;
 		else {
 		    float fllx, flly, furx, fury;
-		    if (sscanf(line+length("%%PageBoundingBox:"), "%f %f %f %f",
+		    if (sec_sscanf(line+length("%%PageBoundingBox:"), "%f %f %f %f",
 			       &fllx, &flly, &furx, &fury) == 4) {
 			page_bb_set = 1;
 			doc->default_page_boundingbox[LLX] = fllx;
@@ -1126,7 +1159,7 @@ newpage:
             CHECK_MALLOCED(doc->pages);
 	}
 	label = ps_gettext(line+length("%%Page:"), &next_char);
-	if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	if (sec_sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
 	if (nextpage == 1) {
 	    ignore = thispage != 1;
 	}
@@ -1164,7 +1197,7 @@ continuepage:
 		/* Do nothing */
 	    } else if (doc->pages[doc->numpages].orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%256s", text);
+		sec_sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->pages[doc->numpages].orientation = PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -1196,11 +1229,11 @@ continuepage:
 		PS_free(cp);
 	    } else if ((page_bb_set == NONE || page_bb_set == ATEND) &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		sscanf(line+length("%%PageBoundingBox:"), "%256s", text);
+		sec_sscanf(line+length("%%PageBoundingBox:"), "%256s", text);
 		if (strcmp(text, "(atend)") == 0) {
 		    page_bb_set = ATEND;
 		} else {
-		    if (sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
+		    if (sec_sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
 			    &(doc->pages[doc->numpages].boundingbox[LLX]),
 			    &(doc->pages[doc->numpages].boundingbox[LLY]),
 			    &(doc->pages[doc->numpages].boundingbox[URX]),
@@ -1210,7 +1243,7 @@ continuepage:
 		      }
 		    else {
 			float fllx, flly, furx, fury;
-			if (sscanf(line+length("%%PageBoundingBox:"),
+			if (sec_sscanf(line+length("%%PageBoundingBox:"),
 				   "%f %f %f %f",
 				   &fllx, &flly, &furx, &fury) == 4) {
 			    if (page_bb_set == NONE) page_bb_set = 1;
@@ -1261,7 +1294,7 @@ continuepage:
 	    /* Do nothing */
 	} else if (iscomment(line+2, "Page:")) {
 	    PS_free(ps_gettext(line+length("%%Page:"), &next_char));
-	    if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	    if (sec_sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
 	    if (!ignore && thispage == nextpage) {
 		if (doc->numpages > 0) {
 		    doc->pages[doc->numpages-1].end = position;
@@ -1288,13 +1321,13 @@ continuepage:
 	    doc->begintrailer = position;
 	    section_len = line_len;
 	} else if (bb_set == ATEND && iscomment(line+2, "BoundingBox:")) {
-	    if (sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
+	    if (sec_sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
 		       &(doc->boundingbox[LLX]),
 		       &(doc->boundingbox[LLY]),
 		       &(doc->boundingbox[URX]),
 		       &(doc->boundingbox[URY])) != 4) {
 		float fllx, flly, furx, fury;
-		if (sscanf(line+length("%%BoundingBox:"), "%f %f %f %f",
+		if (sec_sscanf(line+length("%%BoundingBox:"), "%f %f %f %f",
 			   &fllx, &flly, &furx, &fury) == 4) {
 		    doc->boundingbox[LLX] = fllx;
 		    doc->boundingbox[LLY] = flly;
@@ -1312,14 +1345,14 @@ continuepage:
 	    }
 	} else if (orientation_set == ATEND &&
 		   iscomment(line+2, "Orientation:")) {
-	    sscanf(line+length("%%Orientation:"), "%256s", text);
+	    sec_sscanf(line+length("%%Orientation:"), "%256s", text);
 	    if (strcmp(text, "Portrait") == 0) {
 		doc->orientation = PORTRAIT;
 	    } else if (strcmp(text, "Landscape") == 0) {
 		doc->orientation = LANDSCAPE;
 	    }
 	} else if (page_order_set == ATEND && iscomment(line+2, "PageOrder:")) {
-	    sscanf(line+length("%%PageOrder:"), "%256s", text);
+	    sec_sscanf(line+length("%%PageOrder:"), "%256s", text);
 	    if (strcmp(text, "Ascend") == 0) {
 		doc->pageorder = ASCEND;
 	    } else if (strcmp(text, "Descend") == 0) {
@@ -1328,7 +1361,7 @@ continuepage:
 		doc->pageorder = SPECIAL;
 	    }
 	} else if (pages_set == ATEND && iscomment(line+2, "Pages:")) {
-	    if (sscanf(line+length("%%Pages:"), "%*u %d", &i) == 1) {
+	    if (sec_sscanf(line+length("%%Pages:"), "%*u %d", &i) == 1) {
 		if (page_order_set == NONE) {
 		    if (i == -1) doc->pageorder = DESCEND;
 		    else if (i == 0) doc->pageorder = SPECIAL;
@@ -1354,7 +1387,7 @@ continuepage:
 	preread = 0;
 	if (DSCcomment(line) && iscomment(line+2, "Page:")) {
 	    PS_free(ps_gettext(line+length("%%Page:"), &next_char));
-	    if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
+	    if (sec_sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
 	    if (!ignore && thispage == nextpage) {
 		if (doc->numpages > 0) {
 		    doc->pages[doc->numpages-1].end = position;
@@ -1874,7 +1907,7 @@ static char * readline (fd, lineP, positionP, line_lenP)
       INFMESSAGE(encountered "BeginData:")
       if (FD_LINE_LEN > 100) FD_BUF[100] = '\0';
       text[0] = '\0';
-      if (sscanf(line+length("%%BeginData:"), "%d %*s %100s", &num, text) >= 1) {
+      if (sec_sscanf(line+length("%%BeginData:"), "%d %*s %100s", &num, text) >= 1) {
          if (strcmp(text, "Lines") == 0) {
             INFIMESSAGE(number of lines to skip:,num)
             while (num) {
@@ -1898,7 +1931,7 @@ static char * readline (fd, lineP, positionP, line_lenP)
    else if IS_BEGIN("Binary:") {
       int  num;
       INFMESSAGE(encountered "BeginBinary:")
-      if (sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
+      if (sec_sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
          int read_chunk_size = LINE_CHUNK_SIZE;
          INFIMESSAGE(number of chars to skip:,num)
          while (num>0) {
@@ -1973,7 +2006,7 @@ pscopyuntil(fd, to, begin, end, comment)
          INFMESSAGE(encountered "BeginData:")
          if (FD_LINE_LEN > 100) FD_BUF[100] = '\0';
          text[0] = '\0';
-         if (sscanf(line+length("%%BeginData:"), "%d %*s %100s", &num, text) >= 1) {
+         if (sec_sscanf(line+length("%%BeginData:"), "%d %*s %100s", &num, text) >= 1) {
             if (strcmp(text, "Lines") == 0) {
                INFIMESSAGE(number of lines:,num)
                while (num) {
@@ -1996,7 +2029,7 @@ pscopyuntil(fd, to, begin, end, comment)
       else if IS_BEGIN("Binary:") {
          int  num;
          INFMESSAGE(encountered "BeginBinary:")
-         if (sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
+         if (sec_sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
             int read_chunk_size = LINE_CHUNK_SIZE;
             INFIMESSAGE(number of chars:,num)
             while (num>0) {
@@ -2070,12 +2103,12 @@ pscopydoc(dest_file,src_filename,d,pagelist)
           PS_free(comment);
           continue;
        }
-       sscanf(comment+length("%%Pages:"), "%256s", text);
+       sec_sscanf(comment+length("%%Pages:"), "%256s", text);
        if (strcmp(text, "(atend)") == 0) {
           fputs(comment, dest_file);
           pages_atend = True;
        } else {
-          switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+          switch (sec_sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
              case 1:
                 fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
                 break;
@@ -2110,7 +2143,7 @@ pscopydoc(dest_file,src_filename,d,pagelist)
          PS_free(comment);
          continue;
       }
-      switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+      switch (sec_sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
          case 1:
             fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
             break;
